/*

 Copyright (c) [2023] [pcm]
 [pcm-coordinator] is licensed under Mulan PSL v2.
 You can use this software according to the terms and conditions of the Mulan PSL v2.
 You may obtain a copy of Mulan PSL v2 at:
          http://license.coscl.org.cn/MulanPSL2
 THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 EITHER EXPaRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 See the Mulan PSL v2 for more details.

*/

package ai

import (
	"context"
	"github.com/jinzhu/copier"
	"github.com/pkg/errors"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/helper/xerr"
	error2 "gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/repository/error"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/utils"
	"gitlink.org.cn/jcce-pcm/pcm-participant-modelarts/modelarts"
	"k8s.io/apimachinery/pkg/util/json"

	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/svc"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type ListDataSetLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewListDataSetLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListDataSetLogic {
	return &ListDataSetLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ListDataSetLogic) ListDataSet(req *types.DataSetReq) (resp *types.DataSetResp, err error) {
	// todo: add your logic here and delete this line
	modelartsReq := &modelarts.DataSetReq{}
	err = copier.CopyWithOption(modelartsReq, req, copier.Option{Converters: utils.Converters})
	//err = copier.CopyWithOption(modelartsReq, req, copier.Option{IgnoreEmpty: true, DeepCopy: true, Converters: utils.Converters})
	DataSetResp, err := l.svcCtx.ModelArtsRpc.GetDatasetList(l.ctx, modelartsReq)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrMsg("Failed to get db DataSet list"), "Failed to get db DataSet list err : %v ,req:%+v", err, req)
	}
	marshal, err := json.Marshal(&DataSetResp)
	if err != nil {
		return nil, error2.NewDefaultError(err.Error())
	}
	json.Unmarshal(marshal, &resp)
	err = copier.CopyWithOption(&resp, &DataSetResp, copier.Option{Converters: utils.Converters})
	return resp, nil
}
