/*

 Copyright (c) [2023] [pcm]
 [pcm-coordinator] is licensed under Mulan PSL v2.
 You can use this software according to the terms and conditions of the Mulan PSL v2.
 You may obtain a copy of Mulan PSL v2 at:
          http://license.coscl.org.cn/MulanPSL2
 THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 EITHER EXPaRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 See the Mulan PSL v2 for more details.

*/

package core

import (
	"context"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/zeromicro/go-zero/core/logx"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/svc"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/types"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/models"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/utils"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/utils/httputils"
	"k8s.io/apimachinery/pkg/util/json"
	"math"
	"strconv"
)

type ListDomainResourceLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewListDomainResourceLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListDomainResourceLogic {
	return &ListDomainResourceLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ListDomainResourceLogic) ListDomainResource() (resp *types.DomainResourceResp, err error) {
	//TODO 域信息查询数算集群监控
	resp = &types.DomainResourceResp{}
	var domainResourceModel []models.DomainResource
	sqlStr := "select * from `joint_domain`.domain_cluster"
	l.svcCtx.DbEngin.Raw(sqlStr).Scan(&domainResourceModel)
	utils.Convert(domainResourceModel, &resp.DomainResourceList)
	metrics := &types.ClusterMetrics{}
	//获取指定集群监控数据
	for i := 0; i < len(resp.DomainResourceList); i++ {
		url := fmt.Sprintf("%s/api/v1/resource/getClusterMetrics?clusterName=%s", l.svcCtx.Config.JccScheduleUrl, resp.DomainResourceList[i].ClusterName)
		bytes, err := httputils.HttpGet("GET", url)
		if err != nil {
			logx.WithContext(l.ctx).Errorf("获取云算指定集群监控数据 err: %s", err.Error())
			return nil, err
		}
		json.Unmarshal(bytes, metrics)
		if metrics.Code == 200 {
			//fmt.Print(metrics)
			for _, metricResult := range metrics.Data {
				//集群内存已使用
				if metricResult.MetricName == "cluster_memory_usage_wo_cache" {
					format, ok := metricResult.MetricData.Result[0]["values"].([]interface{})
					if ok {
						for _, val := range format {
							f, ok := val.([]interface{})
							if ok {
								value, _ := f[1].(string)
								v1, err := strconv.ParseFloat(value, 32)
								if err != nil {
									continue
								}
								v2, _ := decimal.NewFromFloat(v1 / (1024 * 1024 * 1024)).Round(2).Float64()
								resp.DomainResourceList[i].MemoryUsage = v2
							} else {
								continue
							}
						}
					} else {
						continue
					}
				}
				//集群内存总量
				if metricResult.MetricName == "cluster_memory_total" {
					format, ok := metricResult.MetricData.Result[0]["values"].([]interface{})
					if ok {
						for _, val := range format {
							f, ok := val.([]interface{})
							if ok {
								value, _ := f[1].(string)
								v1, err := strconv.ParseFloat(value, 32)
								if err != nil {
									continue
								}
								v2, _ := decimal.NewFromFloat(v1 / (1024 * 1024 * 1024)).Round(2).Float64()
								resp.DomainResourceList[i].MemoryTotal = v2
							} else {
								continue
							}
						}
					} else {
						continue
					}
				}
				//集群cpu已使用
				if metricResult.MetricName == "cluster_cpu_usage" {
					format, ok := metricResult.MetricData.Result[0]["values"].([]interface{})
					if ok {
						for _, val := range format {
							f, ok := val.([]interface{})
							if ok {
								value, _ := f[1].(string)
								v1, err := strconv.ParseFloat(value, 32)
								if err != nil {
									continue
								}
								v2, _ := decimal.NewFromFloat(v1).Round(2).Float64()
								resp.DomainResourceList[i].CpuUsage = v2
							} else {
								continue
							}
						}
					} else {
						continue
					}
				}
				//集群cpu总核数
				if metricResult.MetricName == "cluster_cpu_total" {
					format, ok := metricResult.MetricData.Result[0]["values"].([]interface{})
					if ok {
						for _, val := range format {
							f, ok := val.([]interface{})
							if ok {
								value, _ := f[1].(string)
								v1, err := strconv.ParseFloat(value, 32)
								if err != nil {
									continue
								}
								v2, _ := decimal.NewFromFloat(v1).Round(2).Float64()
								resp.DomainResourceList[i].CpuTotal = v2
							} else {
								continue
							}
						}
					} else {
						continue
					}
				}
				//集群容器节点已使用数量
				if metricResult.MetricName == "cluster_pod_running_count" {
					format, ok := metricResult.MetricData.Result[0]["values"].([]interface{})
					if ok {
						for _, val := range format {
							f, ok := val.([]interface{})
							if ok {
								value, _ := f[1].(string)
								v1, err := strconv.ParseFloat(value, 32)
								if err != nil {
									continue
								}
								v2, _ := decimal.NewFromFloat(v1).Round(2).Float64()
								resp.DomainResourceList[i].NodeUsage = v2
							} else {
								continue
							}
						}
					} else {
						continue
					}
				}
				//集群容器节点总量
				if metricResult.MetricName == "cluster_pod_quota" {
					format, ok := metricResult.MetricData.Result[0]["values"].([]interface{})
					if ok {
						for _, val := range format {
							f, ok := val.([]interface{})
							if ok {
								value, _ := f[1].(string)
								v1, err := strconv.ParseFloat(value, 32)
								if err != nil {
									continue
								}
								v2, _ := decimal.NewFromFloat(v1).Round(2).Float64()
								resp.DomainResourceList[i].NodeTotal = v2
							} else {
								continue
							}
						}
					} else {
						continue
					}
				}
				//集群磁盘已使用
				if metricResult.MetricName == "cluster_disk_size_usage" {
					format, ok := metricResult.MetricData.Result[0]["values"].([]interface{})
					if ok {
						for _, val := range format {
							f, ok := val.([]interface{})
							if ok {
								value, _ := f[1].(string)
								v1, err := strconv.ParseFloat(value, 32)
								if err != nil {
									continue
								}
								v2, _ := decimal.NewFromFloat(v1 / (1000 * 1000 * 1000)).Round(2).Float64()
								resp.DomainResourceList[i].DiskUsage = v2
							} else {
								continue
							}
						}
					} else {
						continue
					}
				}
				//集群磁盘总量
				if metricResult.MetricName == "cluster_disk_size_capacity" {
					format, ok := metricResult.MetricData.Result[0]["values"].([]interface{})
					if ok {
						for _, val := range format {
							f, ok := val.([]interface{})
							if ok {
								value, _ := f[1].(string)
								v1, err := strconv.ParseFloat(value, 32)
								if err != nil {
									continue
								}
								v2, _ := decimal.NewFromFloat(v1 / (1000 * 1000 * 1000)).Round(2).Float64()
								resp.DomainResourceList[i].DiskTotal = v2
							} else {
								continue
							}
						}
					} else {
						continue
					}
				}
				if resp.DomainResourceList[i].CpuUsage != 0 && resp.DomainResourceList[i].CpuTotal != 0 {
					resp.DomainResourceList[i].Cpu = math.Ceil((resp.DomainResourceList[i].CpuUsage / resp.DomainResourceList[i].CpuTotal) * 10000 / 100.00)
				}
				if resp.DomainResourceList[i].NodeUsage != 0 && resp.DomainResourceList[i].NodeTotal != 0 {
					resp.DomainResourceList[i].NodeCount = math.Ceil((resp.DomainResourceList[i].NodeUsage / resp.DomainResourceList[i].NodeTotal) * 10000 / 100.00)
				}
				if resp.DomainResourceList[i].DiskUsage != 0 && resp.DomainResourceList[i].DiskTotal != 0 {
					resp.DomainResourceList[i].Disk = math.Ceil((resp.DomainResourceList[i].DiskUsage / resp.DomainResourceList[i].DiskTotal) * 10000 / 100.00)
				}
				if resp.DomainResourceList[i].MemoryUsage != 0 && resp.DomainResourceList[i].MemoryTotal != 0 {
					resp.DomainResourceList[i].Memory = math.Ceil((resp.DomainResourceList[i].MemoryUsage / resp.DomainResourceList[i].MemoryTotal) * 10000 / 100.00)
				}
				resp.DomainResourceList[i].DomainName = resp.DomainResourceList[i].Description
				resp.DomainResourceList[i].ResourceType = "云算"
				resp.DomainResourceList[i].Stack = "kubernetes"
			}
		}
	}
	return resp, nil
}
