/*

 Copyright (c) [2023] [pcm]
 [pcm-coordinator] is licensed under Mulan PSL v2.
 You can use this software according to the terms and conditions of the Mulan PSL v2.
 You may obtain a copy of Mulan PSL v2 at:
          http://license.coscl.org.cn/MulanPSL2
 THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 EITHER EXPaRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 See the Mulan PSL v2 for more details.

*/

package core

import (
	"context"
	"github.com/zeromicro/go-zero/core/logx"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/svc"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/types"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/utils"
)

type ParticipantListLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewParticipantListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ParticipantListLogic {
	return &ParticipantListLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ParticipantListLogic) ParticipantList() (resp *types.ParticipantListResp, err error) {
	var participants []types.Participant

	tx := l.svcCtx.DbEngin.Raw("SELECT ppi.id,ppi.name,ppi.address,ppi.metrics_url ,ti.tenant_name,(CASE ppi.type WHEN '0' THEN '数算' WHEN '1' THEN '智算' WHEN '2' THEN '超算' ELSE '' END) as type_name from sc_participant_phy_info ppi LEFT JOIN sc_tenant_info ti on ti.id = ppi.tenant_id  where ppi.deleted_flag = '0'").Scan(&participants)
	if tx.Error != nil {
		return nil, tx.Error
	}
	result := types.ParticipantListResp{}
	utils.Convert(participants, &result.Participants)
	return &result, nil
}
