/*

 Copyright (c) [2023] [pcm]
 [pcm-coordinator] is licensed under Mulan PSL v2.
 You can use this software according to the terms and conditions of the Mulan PSL v2.
 You may obtain a copy of Mulan PSL v2 at:
          http://license.coscl.org.cn/MulanPSL2
 THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 EITHER EXPaRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 See the Mulan PSL v2 for more details.

*/

package vm

import (
	"context"
	"github.com/jinzhu/copier"
	"github.com/pkg/errors"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/svc"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/types"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/helper/xerr"
	error2 "gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/repository/error"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/utils"
	"gitlink.org.cn/jcce-pcm/pcm-participant-openstack/openstack"
	"k8s.io/apimachinery/pkg/util/json"

	"github.com/zeromicro/go-zero/core/logx"
)

type ListVolumeTypesLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewListVolumeTypesLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListVolumeTypesLogic {
	return &ListVolumeTypesLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ListVolumeTypesLogic) ListVolumeTypes(req *types.ListVolumeTypesReq) (resp *types.ListVolumeTypesResp, err error) {
	// todo: add your logic here and delete this line
	ListVolumeTypesReq := &openstack.ListVolumeTypesReq{}
	err = copier.CopyWithOption(ListVolumeTypesReq, req, copier.Option{Converters: utils.Converters})
	ListVolumeTypesResp, err := l.svcCtx.OpenstackRpc.ListVolumeTypes(l.ctx, ListVolumeTypesReq)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrMsg("Failed to get Volumes list"), "Failed to get db Volumes list err : %v ,req:%+v", err, req)
	}
	marshal, err := json.Marshal(&ListVolumeTypesResp)
	if err != nil {
		return nil, error2.NewDefaultError(err.Error())
	}
	json.Unmarshal(marshal, &resp)
	err = copier.CopyWithOption(&resp, &ListVolumeTypesResp, copier.Option{Converters: utils.Converters})
	return resp, err
}
