/*

 Copyright (c) [2023] [pcm]
 [pcm-coordinator] is licensed under Mulan PSL v2.
 You can use this software according to the terms and conditions of the Mulan PSL v2.
 You may obtain a copy of Mulan PSL v2 at:
          http://license.coscl.org.cn/MulanPSL2
 THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 EITHER EXPaRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 See the Mulan PSL v2 for more details.

*/

package mqs

import (
	"context"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/api/internal/svc"
	"gitlink.org.cn/jcce-pcm/pcm-coordinator/pkg/scheduler"
)

/*
*
 */
type CloudMq struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewCloudMq(ctx context.Context, svcCtx *svc.ServiceContext) *CloudMq {
	return &CloudMq{
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *CloudMq) Consume(val string) error {
	// 接受消息, 根据标签筛选过滤
	cloudScheduler := scheduler.NewCloudScheduler()
	schdl, err := scheduler.NewScheduler(cloudScheduler, val, l.svcCtx.DbEngin, l.svcCtx.ParticipantRpc)
	if err != nil {
		return err
	}

	//检测是否指定了集群列表
	schdl.SpecifyClusters()

	//检测是否指定了nsID
	schdl.SpecifyNsID()

	//通过标签匹配筛选出集群范围
	schdl.MatchLabels()

	//todo 屏蔽原调度算法,因为监控数据暂未上报，临时采用随机调度
	schdl.TempAssign()

	// 存储数据
	err = schdl.SaveToDb()
	if err != nil {
		return err
	}
	return nil
}
